pub struct HoprSessionConfigurator {
pub(crate) id: SessionId,
pub(crate) smgr: Weak<SessionManager<Sender<(DestinationRouting, ApplicationDataOut)>, Sender<IncomingSession>>>,
}Expand description
Allows configuration of one specific HoprSession.
The configurator does not prevent the Session from being closed or the Session manager from being dropped.
Fields
id: SessionIdsmgr: Weak<SessionManager<Sender<(DestinationRouting, ApplicationDataOut)>, Sender<IncomingSession>>>Implementations
Sourceimpl HoprSessionConfigurator
impl HoprSessionConfigurator
Sourcepub async fn ping(&self) -> Result<()>
pub async fn ping(&self) -> Result<()>
Sends a Session Keep-Alive packet over the Session.
NOTE: This usually carries at least 2 SURBs on the HOPR protocol level and can be used for manual SURB balancing.
NOTE: This operation only sends the Session Keep-Alive packet and DOES NOT guarantee the other side has received it.
Sourcepub async fn get_surb_balancer_config(
&self,
) -> Result<Option<SurbBalancerConfig>>
pub async fn get_surb_balancer_config( &self, ) -> Result<Option<SurbBalancerConfig>>
Gets the configuration of the SURB balancer.
Returns an error if the Session is closed, the Session manager is gone.
Returns Ok(None) if the Session has been created without a SURB balancer.
Sourcepub async fn update_surb_balancer_config(
&self,
config: SurbBalancerConfig,
) -> Result<()>
pub async fn update_surb_balancer_config( &self, config: SurbBalancerConfig, ) -> Result<()>
Updates the configuration of the SURB balancer.
Returns an error if the Session is closed, the Session manager is gone, or the Session has been created without a SURB balancer.