HoprTransport

Struct HoprTransport 

pub struct HoprTransport<Chain, Graph, Net> {
Show 13 fields pub(crate) packet_key: OffchainKeypair, pub(crate) chain_key: ChainKeypair, pub(crate) chain_api: Chain, pub(crate) ping: Arc<OnceLock<Pinger>>, pub(crate) network: Arc<OnceLock<Net>>, pub(crate) graph: Graph, pub(crate) path_planner: PathPlanner<MemorySurbStore, Chain, HoprGraphPathSelector<Graph>>, pub(crate) my_multiaddresses: Vec<Multiaddr>, pub(crate) smgr: Arc<SessionManager<Sender<(DestinationRouting, ApplicationDataOut)>>>, pub(crate) session_telemetry_tag_allocator: Arc<dyn TagAllocator + Send + Sync>, pub(crate) probing_tag_allocator: Arc<dyn TagAllocator + Send + Sync>, pub(crate) counters: PeerProtocolCounterRegistry, pub(crate) cfg: HoprProtocolConfig,
}
Interface into the physical transport mechanism allowing all off-chain HOPR-related tasks on the transport.

Fields

packet_key: OffchainKeypairchain_key: ChainKeypairchain_api: Chainping: Arc<OnceLock<Pinger>>network: Arc<OnceLock<Net>>graph: Graphpath_planner: PathPlanner<MemorySurbStore, Chain, HoprGraphPathSelector<Graph>>my_multiaddresses: Vec<Multiaddr>smgr: Arc<SessionManager<Sender<(DestinationRouting, ApplicationDataOut)>>>session_telemetry_tag_allocator: Arc<dyn TagAllocator + Send + Sync>probing_tag_allocator: Arc<dyn TagAllocator + Send + Sync>counters: PeerProtocolCounterRegistrycfg: HoprProtocolConfig

Implementations

Source

impl<Chain, Graph, Net> HoprTransport<Chain, Graph, Net>
where Chain: ChainReadChannelOperations + ChainReadAccountOperations + ChainWriteTicketOperations + ChainKeyOperations + ChainReadTicketOperations + ChainValues + Clone + Send + Sync + 'static, Graph: NetworkGraphView<NodeId = OffchainPublicKey> + NetworkGraphUpdate + NetworkGraphWrite<NodeId = OffchainPublicKey> + NetworkGraphTraverse<NodeId = OffchainPublicKey> + Clone + Send + Sync + 'static, <Graph as NetworkGraphView>::Observed: EdgeObservableRead + Send, <Graph as NetworkGraphTraverse>::Observed: EdgeObservableRead + Send + 'static, <Graph as NetworkGraphWrite>::Observed: EdgeObservableWrite + Send, Net: NetworkView + NetworkStreamControl + Clone + Send + Sync + 'static,

pub fn new( identity: (&ChainKeypair, &OffchainKeypair), resolver: Chain, graph: Graph, my_multiaddresses: Vec<Multiaddr>, cfg: HoprProtocolConfig, ) -> Result<Self>

pub async fn run_relay<T, TFact, Ct>( &self, cover_traffic: Ct, network: Net, network_process: BoxedProcessFn, ticket_events: T, ticket_factory: TFact, on_incoming_session: Sender<IncomingSession>, ) -> Result<(HoprSocket<BoxStream<'static, ApplicationDataIn>, Sender<(DestinationRouting, ApplicationDataOut)>>, AbortableList<HoprTransportProcess>)>
where T: Sink<TicketEvent> + Clone + Send + Unpin + 'static, T::Error: Error + Clone + Send, Ct: ProbingTrafficGeneration + CoverTrafficGeneration + Send + Sync + 'static, TFact: TicketFactory + Clone + Send + Sync + 'static,

Execute all processes of the HoprTransport object as a Relay node.

Relay nodes run the full packet pipeline including incoming ticket/acknowledgement processing and require a [futures::Sink] for ticket events as well as an on_incoming_session channel from the SessionManager (they can accept incoming sessions).

pub async fn run_exit<TFact, Ct>( &self, cover_traffic: Ct, network: Net, network_process: BoxedProcessFn, ticket_factory: TFact, on_incoming_session: Sender<IncomingSession>, ) -> Result<(HoprSocket<BoxStream<'static, ApplicationDataIn>, Sender<(DestinationRouting, ApplicationDataOut)>>, AbortableList<HoprTransportProcess>)>
where Ct: ProbingTrafficGeneration + CoverTrafficGeneration + Send + Sync + 'static, TFact: TicketFactory + Clone + Send + Sync + 'static,

Execute all processes of the HoprTransport object as an Exit (destination) node.

Exit nodes do not process tickets but keep the incoming acknowledgement pipeline running and can accept incoming sessions via SessionManager.

pub async fn run_entry<TFact, Ct>( &self, cover_traffic: Ct, network: Net, network_process: BoxedProcessFn, ticket_factory: TFact, ) -> Result<(HoprSocket<BoxStream<'static, ApplicationDataIn>, Sender<(DestinationRouting, ApplicationDataOut)>>, AbortableList<HoprTransportProcess>)>
where Ct: ProbingTrafficGeneration + CoverTrafficGeneration + Send + Sync + 'static, TFact: TicketFactory + Clone + Send + Sync + 'static,

Execute all processes of the HoprTransport object as an Entry (source) node.

Entry nodes do not process tickets, do not start the incoming acknowledgement pipeline, and do not accept incoming sessions — therefore, they require neither a ticket_events sink nor an on_incoming_session channel.

pub(crate) async fn run_inner<T, TFact, Ct>( &self, role: NodeType, cover_traffic: Ct, network: Net, network_process: BoxedProcessFn, ticket_events: T, ticket_factory: TFact, on_incoming_session: Option<Sender<IncomingSession>>, ) -> Result<(HoprSocket<BoxStream<'static, ApplicationDataIn>, Sender<(DestinationRouting, ApplicationDataOut)>>, AbortableList<HoprTransportProcess>)>
where T: Sink<TicketEvent> + Clone + Send + Unpin + 'static, T::Error: Error + Clone + Send, Ct: ProbingTrafficGeneration + CoverTrafficGeneration + Send + Sync + 'static, TFact: TicketFactory + Clone + Send + Sync + 'static,

Internal worker driving all node-type variants of HoprTransport::run_*.

Branches on role:

pub async fn ping( &self, peer: &OffchainPublicKey, ) -> Result<(Duration, <Graph as NetworkGraphView>::Observed)>

pub async fn new_session( &self, destination: Address, target: SessionTarget, cfg: SessionClientConfig, ) -> Result<(HoprSession, HoprSessionConfigurator)>

pub async fn listening_multiaddresses(&self) -> Vec<Multiaddr>

pub fn announceable_multiaddresses(&self) -> Vec<Multiaddr>

pub fn graph(&self) -> &Graph

Returns a reference to the network graph.

pub fn local_multiaddresses(&self) -> Vec<Multiaddr>

pub async fn network_observed_multiaddresses( &self, peer: &OffchainPublicKey, ) -> Vec<Multiaddr>

pub async fn network_health(&self) -> Health

pub async fn network_connected_peers(&self) -> Result<Vec<OffchainPublicKey>>

pub fn network_peer_observations( &self, peer: &OffchainPublicKey, ) -> Option<<Graph as NetworkGraphView>::Observed>

pub async fn all_network_peers( &self, minimum_score: f64, ) -> Result<Vec<(OffchainPublicKey, <Graph as NetworkGraphView>::Observed)>>

Get connected peers with quality higher than some value.